#!/bin/bash

yes_no_list=" yes no "
ERR_CNT=0
TST_CNT=0
NR_LOOPS=10
NR_ILOOPS=10

# QA: This bug will not be reproducible in the above test. Fixes to txtest and to
# the store will prevent the "message shrinking" bug from occurring, and hence
# this bug.
# 
# It may be tested using a purpose-written test:
# 
# 1. Start a broker using the --staging-threshold set at an intermediate value
# (say 50 - 1000 bytes).
# 
# 2. Start a short run (100 - 1000 messages) of durable/transient messages which
# are larger than the threshold - this will force flow-to-disk to occur and will
# validate the read pipeline, setting it up to keep up with the writes as long as
# it is valid.
# 
# 3. On the *same queue*, now submit enough messages that are smaller than the
# threshold that the entire journal will be overwritten at least once. As soon as
# the file containing the last read is overwritten by the writes, the read
# pipeline will invalidate itself.
# 
# 4. Repeat steps 2 and 3 several times.
# 
# This bug has only a probability of occurring with this test - there needs to be
# at least one read async io in-flight at the moment the write pipeline
# invalidates the read pipeline. When the read pipeline is revalidated, this
# event - left over from the last test - is the first to be processed and causes
# the error message (JERR_FCNTL_CMPLOFFSOVFL).
# 
# Either txtest or perftest could be used, so long as the number of queues
# remains consistent for the test.

# source functions

if [ -r ./mrg_test_functions.sh ]; then
  source ./mrg_test_functions.sh
else
  echo "Test cannot continue because of missing part!"
  exit 254
fi

DATE_TIME_START=$(date +%Y%m%d_%H%M%S)
TEMP_FILE=tmp_${DATE_TIME_START}
TEMP_FILE_GLOB=tmp_*

QPIDD_DIR=${TEST_PARAM_RHTS_TEST_QPIDD_DIR:-/usr/sbin}
QPIDD_DATADIR="${PWD}/rhts_qpidd/${DATE_TIME_START}"

VERBOSE_LEVEL=1
mrg_gen_my_rand_in_range 64000 64004
QPIDD_PORT=${MY_RAND}
QPIDD_TEST_PARAMS_COMMON="--auth no --staging-threshold 500"
if [ "${VERBOSE_LEVEL}" -gt "10" ]; then
  QPIDD_TEST_PARAMS="${QPIDD_TEST_PARAMS_COMMON} --log-enable trace+"
elif [ "${VERBOSE_LEVEL}" -gt "7" ]; then
  QPIDD_TEST_PARAMS="${QPIDD_TEST_PARAMS_COMMON} --log-enable debug+"
else
  QPIDD_TEST_PARAMS="${QPIDD_TEST_PARAMS_COMMON} --log-enable info+"
fi
QPIDD_TRANSCRIPT="qpidd.transcript.log"
QPIDD_LOCK=qpidd.lock
QPIDD_ECODE=qpidd.ecode

TXTEST_PARAMS_COMMON="--log-enable debug+ "
TXTEST_PARAMS_COMMON="--log-enable info+ "


mrg_rm $QPIDD_LOCK $QPIDD_ECODE $QPIDD_TRANSCRIPT $QPIDD_DATADIR txtest.log 

# stop all qpidd brokers
# ---------------------------------------------------------------------------
lognl "Stopping all running qpidd instances..."
mrg_qpidd_stop
ecode="$?"
if [ "${ecode}" == "0" -a "${QPIDD_INST_CNT}" == "0" ]; then
  lognl ".qpidd stopped ok"
else
  lognl ".ERROR:qpidd didn't stop " \
        "(ecode:${ecode}|qpidd_int_cnt:${QPIDD_INST_CNT})"
  let "ERR_CNT++"
fi

# launch & wait for settle qpidd

# create qpidd data journal storage
# -------------------------------------------------------------------------
mrg_rm ${QPIDD_DATADIR}
mkdir -p ${QPIDD_DATADIR}
QPIDD_DATADIR_JOURNAL=$(mktemp -d ${QPIDD_DATADIR}/broker.XXXX)

# start qpidd
# -------------------------------------------------------------------------
mrg_qpidd_start_normal ${QPIDD_PORT} ${QPIDD_DATADIR_JOURNAL} \
                        ${QPIDD_TRANSCRIPT} ${QPIDD_TEST_PARAMS}

# wait for qpidd ready
# -------------------------------------------------------------------------
mrg_qpidd_wait_on_settle ${QPIDD_TRANSCRIPT} "Listening on TCP port" \
                          ${QPIDD_LOCK} 20
if [ "$?" == "0" ]; then
  lognl ".qpidd settled"
else
  lognl ".ERROR:qpidd not started"
  let "ERR_CNT++"
fi


for ((i=0;i<${NR_LOOPS};i++)); do
  for i_durable in ${yes_no_list}; do
  
    for i_dtx in ${yes_no_list}; do
      
      for ((j=0;j<${NR_ILOOPS};j++)); do
      
        lognl "1st txtest > threshold (i=${i}, durable=${i_durable}, dtx:${i_dtx}, j=${j})"
        lognl "1st txtest > threshold (i=${i}, durable=${i_durable}, dtx:${i_dtx}, j=${j})" >> txtest.log
        txtest --messages-per-tx 10 --tx-count 1000 --total-messages 100 \
          -p ${QPIDD_PORT} --dtx ${i_dtx} --size 2000 --queues 4 \
          --durable ${i_durable} ${TXTEST_PARAMS_COMMON} >> txtest.log
        ecode="$?"
        echo "run $i [durable=${i_durable}] - ecode=${ecode}"
        if [ ${ecode} != 0 ]; then
          let "ERR_CNT++"
        fi
        
        # verify qpidd is up and running
        # -------------------------------------------------------------------------
        lognl ".qpidd status check"
        mrg_qpidd_status
        if [ -e ${QPIDD_LOCK} -a \
            "${QPIDD_INST_CNT}" == "1" -a \
            "${QPIDD_INST_PORT}" == "${QPIDD_INST_PORT}" ]; then
          lognl "..qpidd running fine on port ${QPIDD_INST_PORT} " \
                "(${QPIDD_INST_CNT} instance[s])"
        else
          lognl "..ERROR:qpidd not running fine! (pid[s]:${QPIDD_INST_PID}, " \
                "port[s]:${QPIDD_INST_PORT}, inst_cnt:${QPIDD_INST_CNT})"
          let "ERR_CNT++"
        fi
        
        
        lognl "1st txtest < threshold (i=${i}, durable=${i_durable}, dtx:${i_dtx}, j=${j})"
        lognl "1st txtest < threshold (i=${i}, durable=${i_durable}, dtx:${i_dtx}, j=${j})" >> txtest.log
        txtest --messages-per-tx 10 --tx-count 100000 --total-messages 10000 \
          -p ${QPIDD_PORT} --dtx ${i_dtx} --size 48 --queues 4 \
          --durable ${i_durable} ${TXTEST_PARAMS_COMMON} >> txtest.log
        ecode="$?"
        echo "run $i [durable=${i_durable}] - ecode=${ecode}"
        if [ ${ecode} != 0 ]; then
          let "ERR_CNT++"
        fi
        
        let "TST_CNT++"
        
        # verify qpidd is up and running
        # -------------------------------------------------------------------------
        lognl ".qpidd status check"
        mrg_qpidd_status
        if [ -e ${QPIDD_LOCK} -a \
            "${QPIDD_INST_CNT}" == "1" -a \
            "${QPIDD_INST_PORT}" == "${QPIDD_INST_PORT}" ]; then
          lognl "..qpidd running fine on port ${QPIDD_INST_PORT} " \
                "(${QPIDD_INST_CNT} instance[s])"
        else
          lognl "..ERROR:qpidd not running fine! (pid[s]:${QPIDD_INST_PID}, " \
                "port[s]:${QPIDD_INST_PORT}, inst_cnt:${QPIDD_INST_CNT})"
          let "ERR_CNT++"
        fi
      
        [ "${QPIDD_INST_CNT}" == "0" ] && break
      
      done
      [ "${QPIDD_INST_CNT}" == "0" ] && break
    done
    [ "${QPIDD_INST_CNT}" == "0" ] && break
  done
  [ "${QPIDD_INST_CNT}" == "0" ] && break
done


echo "done (err_cnt=${ERR_CNT})"

# eof

